@@ -7,8 +7,9 @@
 // $Source: src/binops/checked_div.rs $
 // $Revision: 2022-01-02T11:24:00+01:00 $
 
-use crate::binops::div::div;
-use crate::Decimal;
+use fpdec_core::MAX_N_FRAC_DIGITS;
+
+use crate::{binops::div_rounded::checked_div_rounded, normalize, Decimal};
 
 /// Checked division.
 /// Computes `self / rhs`.
@@ -22,7 +23,7 @@ pub trait CheckedDiv<Rhs = Self> {
 }
 
 impl CheckedDiv<Decimal> for Decimal {
-    type Output = Option<Decimal>;
+    type Output = Option<Self>;
 
     fn checked_div(self, rhs: Decimal) -> Self::Output {
         if rhs.eq_zero() {
@@ -34,14 +35,19 @@ impl CheckedDiv<Decimal> for Decimal {
         if rhs.eq_one() {
             return Some(self);
         }
-        match div(self.coeff, self.n_frac_digits, rhs.coeff, rhs.n_frac_digits)
-        {
-            Ok((coeff, n_frac_digits)) => Some(Self {
-                coeff,
-                n_frac_digits,
-            }),
-            Err(_) => None,
-        }
+        let mut n_frac_digits = MAX_N_FRAC_DIGITS;
+        let mut coeff = checked_div_rounded(
+            self.coeff,
+            self.n_frac_digits,
+            rhs.coeff,
+            rhs.n_frac_digits,
+            n_frac_digits,
+        )?;
+        normalize(&mut coeff, &mut n_frac_digits);
+        Some(Self {
+            coeff,
+            n_frac_digits,
+        })
     }
 }
 
@@ -60,11 +66,11 @@ mod checked_div_decimal_tests {
         let z = x.checked_div(y).unwrap();
         assert_eq!(z.coefficient(), -85);
         assert_eq!(z.n_frac_digits(), 1);
-        let x = Decimal::new_raw(17, 31);
+        let x = Decimal::new_raw(17, 17);
         let y = Decimal::new_raw(2, 0);
         let z = x.checked_div(y).unwrap();
         assert_eq!(z.coefficient(), 85);
-        assert_eq!(z.n_frac_digits(), 32);
+        assert_eq!(z.n_frac_digits(), 18);
         let x = Decimal::new_raw(12345678901234567890, 2);
         let y = Decimal::new_raw(244140625, 6);
         let z = x.checked_div(y).unwrap();
@@ -72,6 +78,15 @@ mod checked_div_decimal_tests {
         assert_eq!(z.n_frac_digits(), 7);
     }
 
+    #[test]
+    fn test_checked_div_frac_limit_exceeded() {
+        let x = Decimal::new_raw(17, 1);
+        let y = Decimal::new_raw(3, 0);
+        let z = x.checked_div(y).unwrap();
+        assert_eq!(z.coefficient(), 566666666666666667);
+        assert_eq!(z.n_frac_digits(), 18);
+    }
+
     #[test]
     fn test_checked_div_by_one() {
         let x = Decimal::new_raw(17, 5);
@@ -91,18 +106,20 @@ mod checked_div_decimal_tests {
         assert!(z.is_none());
     }
 
+    // corner case where divident * shift overflows, but result doesn't
     #[test]
-    fn test_checked_div_frac_limit_exceeded() {
-        let x = Decimal::new_raw(17, 1);
-        let y = Decimal::new_raw(3, 0);
-        let z = x.checked_div(y);
-        assert!(z.is_none());
+    fn test_checked_div_stepwise() {
+        let x = Decimal::new_raw(mul_pow_ten(17, 17), 0);
+        let y = Decimal::new_raw(20498, 5);
+        let z = x.checked_div(y).unwrap();
+        assert_eq!(z.coefficient(), 8293492048004683383744755585910820568_i128);
+        assert_eq!(z.n_frac_digits(), 18);
     }
 
     #[test]
     fn test_checked_div_overflow() {
         let x = Decimal::new_raw(mul_pow_ten(17, 20), 0);
-        let y = Decimal::new_raw(2, 19);
+        let y = Decimal::new_raw(2, 18);
         let z = x.checked_div(y);
         assert!(z.is_none());
     }
@@ -128,7 +145,7 @@ macro_rules! impl_div_decimal_and_int {
     ($($t:ty),*) => {
         $(
         impl CheckedDiv<$t> for Decimal {
-            type Output = Option<Decimal>;
+            type Output = Option<Self>;
 
             fn checked_div(self, rhs: $t) -> Self::Output {
                 if rhs == 0 {
@@ -140,14 +157,19 @@ macro_rules! impl_div_decimal_and_int {
                 if rhs == 1 {
                     return Some(self);
                 }
-                match div(self.coeff, self.n_frac_digits, rhs as i128, 0)
-                {
-                    Ok((coeff, n_frac_digits)) => Some(Self {
-                        coeff,
-                        n_frac_digits,
-                    }),
-                    Err(_) => None,
-                }
+                let mut n_frac_digits = MAX_N_FRAC_DIGITS;
+                let mut coeff = checked_div_rounded(
+                    self.coeff,
+                    self.n_frac_digits,
+                    rhs as i128,
+                    0,
+                    n_frac_digits,
+                )?;
+                normalize(&mut coeff, &mut n_frac_digits);
+                Some(Self {
+                    coeff,
+                    n_frac_digits,
+                })
             }
         }
 
@@ -167,14 +189,19 @@ macro_rules! impl_div_decimal_and_int {
                         n_frac_digits: 0
                     });
                 }
-                match div(self as i128, 0, rhs.coeff, rhs.n_frac_digits)
-                {
-                    Ok((coeff, n_frac_digits)) => Some(Decimal {
-                        coeff,
-                        n_frac_digits,
-                    }),
-                    Err(_) => None,
-                }
+                let mut n_frac_digits = MAX_N_FRAC_DIGITS;
+                let mut coeff = checked_div_rounded(
+                    self as i128,
+                    0,
+                    rhs.coeff,
+                    rhs.n_frac_digits,
+                    n_frac_digits,
+                )?;
+                normalize(&mut coeff, &mut n_frac_digits);
+                Some(Decimal {
+                    coeff,
+                    n_frac_digits,
+                })
             }
         }
         )*
@@ -307,6 +334,24 @@ mod checked_div_integer_tests {
         assert_eq!(z.n_frac_digits(), 0);
     }
 
+    #[test]
+    fn test_checked_div_int_by_decimal_frac_limit_exceeded() {
+        let x = 3_i8;
+        let y = Decimal::new_raw(17, 2);
+        let z = CheckedDiv::checked_div(x, y).unwrap();
+        assert_eq!(z.coefficient(), 17647058823529411765);
+        assert_eq!(z.n_frac_digits(), 18);
+    }
+
+    #[test]
+    fn test_checked_div_decimal_by_int_frac_limit_exceeded() {
+        let x = Decimal::new_raw(17, 12);
+        let y = 3_u8;
+        let z = x.checked_div(y).unwrap();
+        assert_eq!(z.coefficient(), 5666667);
+        assert_eq!(z.n_frac_digits(), 18);
+    }
+
     #[test]
     fn test_checked_div_decimal_by_int_zero() {
         let x = Decimal::new_raw(17, 5);
@@ -323,26 +368,10 @@ mod checked_div_integer_tests {
         assert!(z.is_none());
     }
 
-    #[test]
-    fn test_checked_div_decimal_by_int_frac_limit_exceeded() {
-        let x = Decimal::new_raw(17, 12);
-        let y = 3_u8;
-        let z = x.checked_div(y);
-        assert!(z.is_none());
-    }
-
-    #[test]
-    fn test_checked_div_int_by_decimal_frac_limit_exceeded() {
-        let x = 3_i8;
-        let y = Decimal::new_raw(17, 2);
-        let z = CheckedDiv::checked_div(x, y);
-        assert!(z.is_none());
-    }
-
     #[test]
     fn test_checked_div_int_by_decimal_overflow() {
         let x = mul_pow_ten(17, 20);
-        let y = Decimal::new_raw(2, 19);
+        let y = Decimal::new_raw(2, 18);
         let z = CheckedDiv::checked_div(x, y);
         assert!(z.is_none());
     }
