@@ -7,8 +7,7 @@
 // $Source: src/binops/checked_rem.rs $
 // $Revision: 2022-01-02T11:24:00+01:00 $
 
-use crate::binops::rem::rem;
-use crate::Decimal;
+use crate::{binops::rem::rem, Decimal};
 
 /// Checked remainder.
 /// Computes `self % rhs`.
