@@ -103,7 +103,7 @@ mod cmp_decimals_tests {
     fn test_eq_same_n_frac_digits() {
         let x = Decimal::new_raw(178, 1);
         assert!(x.eq(&x));
-        let y = x.clone();
+        let y = x;
         assert!(x.eq(&y));
         assert_eq!(x, y);
         assert_eq!(y, x);
