@@ -7,11 +7,14 @@
 // $Source: src/binops/div_rounded.rs $
 // $Revision: 2022-01-02T11:24:00+01:00 $
 
-use core::cmp::{min, Ordering};
+use core::cmp::Ordering;
 
-use fpdec_core::{checked_mul_pow_ten, magnitude, ten_pow};
+use fpdec_core::{
+    checked_mul_pow_ten, i128_div_rounded, i128_shifted_div_rounded, ten_pow,
+    MAX_N_FRAC_DIGITS,
+};
 
-use crate::{rounding::div_i128_rounded, Decimal, DecimalError};
+use crate::{Decimal, DecimalError};
 
 const MAGN_I128_MAX: u8 = 38;
 
@@ -48,51 +51,33 @@ pub trait DivRounded<Rhs = Self> {
     fn div_rounded(self, rhs: Rhs, n_frac_digits: u8) -> Self::Output;
 }
 
-#[inline]
-fn div_rounded(
+pub(crate) fn checked_div_rounded(
     divident_coeff: i128,
     divident_n_frac_digits: u8,
     divisor_coeff: i128,
     divisor_n_frac_digits: u8,
     n_frac_digits: u8,
-) -> i128 {
+) -> Option<i128> {
     let mut shift = n_frac_digits + divisor_n_frac_digits;
     match divident_n_frac_digits.cmp(&shift) {
         Ordering::Equal => {
-            div_i128_rounded(divident_coeff, divisor_coeff, None)
+            Some(i128_div_rounded(divident_coeff, divisor_coeff, None))
         }
         Ordering::Less => {
             // divident coeff needs to be shifted
             shift -= divident_n_frac_digits;
+            // 0 < shift <= 36
             if let Some(shifted_divident) =
                 checked_mul_pow_ten(divident_coeff, shift)
             {
-                div_i128_rounded(shifted_divident, divisor_coeff, None)
+                Some(i128_div_rounded(shifted_divident, divisor_coeff, None))
             } else {
-                let magn_divident = magnitude(divident_coeff);
-                let magn_shifted_divident = magn_divident + shift;
-                if (magn_shifted_divident - magnitude(divisor_coeff))
-                    > MAGN_I128_MAX
-                {
-                    panic!("{}", DecimalError::InternalOverflow);
-                }
-                let mut coeff = divident_coeff / divisor_coeff;
-                let mut rem = divident_coeff % divisor_coeff;
-                let mut rem_shift = shift;
-                let mut step_shift =
-                    min(rem_shift, MAGN_I128_MAX - magnitude(rem));
-                while step_shift < rem_shift {
-                    coeff *= ten_pow(step_shift);
-                    rem *= ten_pow(step_shift);
-                    coeff += rem / divisor_coeff;
-                    rem %= divisor_coeff;
-                    rem_shift -= step_shift;
-                    step_shift = min(rem_shift, MAGN_I128_MAX - magnitude(rem));
-                }
-                coeff *= ten_pow(step_shift);
-                rem *= ten_pow(step_shift);
-                coeff += div_i128_rounded(rem, divisor_coeff, None);
-                coeff
+                i128_shifted_div_rounded(
+                    divident_coeff,
+                    shift,
+                    divisor_coeff,
+                    None,
+                )
             }
         }
         Ordering::Greater => {
@@ -103,13 +88,13 @@ fn div_rounded(
             // thus avoiding i128 overflow.
             // divident_n_frac_digits > shift
             shift = divident_n_frac_digits - shift;
-            // shift < divident_n_frac_digits => shift < 38 => ten_pow(shift)
+            // shift < divident_n_frac_digits => shift < 18 => ten_pow(shift)
             // is safe
-            div_i128_rounded(
+            Some(i128_div_rounded(
                 divident_coeff / divisor_coeff,
                 ten_pow(shift),
                 None,
-            )
+            ))
         }
     }
 }
@@ -118,22 +103,28 @@ impl DivRounded<Decimal> for Decimal {
     type Output = Self;
 
     fn div_rounded(self, rhs: Decimal, n_frac_digits: u8) -> Self::Output {
+        if n_frac_digits > MAX_N_FRAC_DIGITS {
+            panic!("{}", DecimalError::MaxNFracDigitsExceeded);
+        }
         if rhs.eq_zero() {
             panic!("{}", DecimalError::DivisionByZero);
         }
         if self.eq_zero() {
             return Self::ZERO;
         }
-        let coeff = div_rounded(
+        if let Some(coeff) = checked_div_rounded(
             self.coeff,
             self.n_frac_digits,
             rhs.coeff,
             rhs.n_frac_digits,
             n_frac_digits,
-        );
-        Self::Output {
-            coeff,
-            n_frac_digits,
+        ) {
+            Self::Output {
+                coeff,
+                n_frac_digits,
+            }
+        } else {
+            panic!("{}", DecimalError::InternalOverflow);
         }
     }
 }
@@ -193,7 +184,7 @@ mod div_rounded_decimal_tests {
         let z = x.div_rounded(y, 3);
         assert_eq!(z.coefficient(), 0);
         assert_eq!(z.n_frac_digits(), 0);
-        let z = x.div_rounded(y, 29);
+        let z = x.div_rounded(y, 18);
         assert_eq!(z.coefficient(), 0);
         assert_eq!(z.n_frac_digits(), 0);
     }
@@ -214,8 +205,8 @@ mod div_rounded_decimal_tests {
     // corner case: shifting divident overflows, stepwise algorithm must be used
     #[test]
     fn test_div_rounded_stepwise() {
-        let x = Decimal::new_raw(13, 1);
-        let y = Decimal::new_raw(20, 29);
+        let x = Decimal::new_raw(mul_pow_ten(13, 11), 1);
+        let y = Decimal::new_raw(20, 18);
         let z = x.div_rounded(y, 10);
         assert_eq!(z.coefficient(), 65000000000000000000000000000000000000);
         assert_eq!(z.n_frac_digits(), 10);
@@ -269,16 +260,19 @@ macro_rules! impl_div_rounded_decimal_and_int {
                 if self.eq_zero() {
                     return Self::ZERO;
                 }
-                let coeff = div_rounded(
+                if let Some(coeff) = checked_div_rounded(
                     self.coeff,
                     self.n_frac_digits,
                     rhs as i128,
                     0_u8,
                     n_frac_digits,
-                );
-                Self::Output {
-                    coeff,
-                    n_frac_digits,
+                ) {
+                    Self::Output {
+                        coeff,
+                        n_frac_digits,
+                    }
+                } else {
+                    panic!("{}", DecimalError::InternalOverflow);
                 }
             }
         }
@@ -329,16 +323,19 @@ macro_rules! impl_div_rounded_decimal_and_int {
                 if self == 0 {
                     return Decimal::ZERO;
                 }
-                let coeff = div_rounded(
+                if let Some(coeff) = checked_div_rounded(
                     self as i128,
                     0_u8,
                     rhs.coeff,
                     rhs.n_frac_digits,
                     n_frac_digits,
-                );
-                Self::Output {
-                    coeff,
-                    n_frac_digits,
+                ) {
+                    Self::Output {
+                        coeff,
+                        n_frac_digits,
+                    }
+                } else {
+                    panic!("{}", DecimalError::InternalOverflow);
                 }
             }
         }
@@ -616,16 +613,19 @@ macro_rules! impl_div_rounded_int_and_int {
                 if self == 0 {
                     return Decimal::ZERO;
                 }
-                let coeff = div_rounded(
+                if let Some(coeff) = checked_div_rounded(
                     self as i128,
                     0_u8,
                     rhs as i128,
                     0_u8,
                     n_frac_digits,
-                );
-                Self::Output {
-                    coeff,
-                    n_frac_digits,
+                ) {
+                    Self::Output {
+                        coeff,
+                        n_frac_digits,
+                    }
+                } else {
+                    panic!("{}", DecimalError::InternalOverflow);
                 }
             }
         }
