@@ -5,10 +5,11 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: src/from_float.rs $
-// $Revision: 2022-01-02T11:24:00+01:00 $
+// $Revision: 2022-03-29T20:45:34+02:00 $
 
 use core::convert::TryFrom;
-use fpdec_core::{magnitude, MAX_N_FRAC_DIGITS};
+
+use fpdec_core::{i128_div_mod_floor, i128_magnitude, MAX_N_FRAC_DIGITS};
 
 use crate::{normalize, Decimal, DecimalError};
 
@@ -80,18 +81,17 @@ fn approx_rational(divident: i128, divisor: i128) -> (i128, u8) {
         return (0, 0);
     }
     let mut n_frac_digits = 0_u8;
-    let mut coeff = divident / divisor;
-    let mut rem = divident % divisor;
-    let mut magn_coeff = magnitude(coeff);
+    let (mut coeff, mut rem) = i128_div_mod_floor(divident, divisor);
+    let mut magn_coeff = i128_magnitude(coeff);
     while rem != 0
         && n_frac_digits < MAX_N_FRAC_DIGITS
         && magn_coeff < MAGN_I128_MAX - 1
     {
-        // rem < divisor
+        // 0 < rem < divisor
         rem *= 10;
-        // rem < 10 * divisor
+        // 0 < rem < 10 * divisor
         let quot = rem / divisor;
-        // quot < 10
+        // 0 <= quot < 10
         rem %= divisor;
         n_frac_digits += 1;
         magn_coeff += 1;
@@ -114,6 +114,7 @@ fn approx_rational(divident: i128, divisor: i128) -> (i128, u8) {
 impl TryFrom<f32> for Decimal {
     type Error = DecimalError;
 
+    //noinspection DuplicatedCode
     /// Tries to convert a `f32` value `f` into a `Decimal`.
     ///
     /// Returns the value representable as a `Decimal` which is nearest to `f`,
@@ -171,6 +172,7 @@ impl TryFrom<f32> for Decimal {
 impl TryFrom<f64> for Decimal {
     type Error = DecimalError;
 
+    //noinspection DuplicatedCode
     /// Tries to convert a `f64` value `f` into a `Decimal`.
     ///
     /// Returns the value representable as a `Decimal` which is nearest to `f`,
@@ -190,7 +192,7 @@ impl TryFrom<f64> for Decimal {
     /// let d = Decimal::try_from(-289.5_f64)?;
     /// assert_eq!(d.to_string(), "-289.5");
     /// let d = Decimal::try_from(37.0005003_f64)?;
-    /// assert_eq!(d.to_string(), "37.000500299999998787825461477041244507");
+    /// assert_eq!(d.to_string(), "37.000500299999998788");
     /// # Ok(()) }
     /// ```
     fn try_from(f: f64) -> Result<Self, Self::Error> {
@@ -261,9 +263,9 @@ mod tests {
     #[test]
     fn test_decimal_from_f32() {
         let test_data = [
-            (-289.5, -2895, 1),
-            (-0.5005, -500500023365020751953125, 24),
-            (37.0005003, 37000499725341796875, 18),
+            (-289.5_f32, -2895, 1),
+            (-0.5005_f32, -500500023365020752, 18),
+            (37.000503_f32, 370005035400390625, 16),
         ];
         check_from_float::<f32>(&test_data);
     }
@@ -284,22 +286,18 @@ mod tests {
     #[test]
     fn test_decimal_from_f64() {
         let test_data = [
-            (
-                -28900.000000005,
-                -28900000000004998582880944013595581054,
-                33,
-            ),
-            (-5e-7, -4999999999999999773740559129431, 37),
+            (-28900.000000005_f64, -28900000000004998582881, 18),
+            (-5e-7, -5, 7),
             (1.004e-127, 0, 0),
-            (1.0005, 1000499999999999944932937978592235595, 36),
-            (37.0005000033, 37000500003299997331396298250183463097, 36),
+            (1.0005, 1000499999999999945, 18),
+            (37.0005000033, 37000500003299997331, 18),
         ];
         check_from_float::<f64>(&test_data);
     }
 
     #[test]
     fn test_fail_overflow() {
-        let f = 5.839e38;
+        let f = 5.839e38_f64;
         let res = Decimal::try_from(f);
         assert!(res.is_err());
         let err = res.unwrap_err();
