@@ -7,8 +7,7 @@
 // $Source: src/traits.rs $
 // $Revision: 2022-01-02T11:24:00+01:00 $
 
-use core::ops::Add;
-use core::str::FromStr;
+use core::{ops::Add, str::FromStr};
 
 use num_traits::{Num, One, Zero};
 
